@@ -642,19 +642,19 @@ class SQLAlchemyDatabase:
             # Connection tuning for bulk operations
             cursor.execute("SET synchronous_commit = OFF")
 
-            # Precompute ON CONFLICT clause based on mode
+            # Precompute conflict handling strategy based on mode
             conflict_cols = ", ".join(f'"{c}"' for c in pks)
+            
+            # For UPDATE mode, we'll use DELETE + INSERT optimization instead of ON CONFLICT
             if mode == "RAISE":
+                use_on_conflict = True
                 on_conflict = ""
             elif mode == "IGNORE":
+                use_on_conflict = True
                 on_conflict = f"ON CONFLICT ({conflict_cols}) DO NOTHING"
-            else:  # UPDATE
+            else:  # UPDATE mode - use DELETE + INSERT optimization
+                use_on_conflict = False
                 non_pk_cols = [c for c in cols if c not in pks]
-                if non_pk_cols:
-                    set_clause = ", ".join(f'"{c}" = EXCLUDED."{c}"' for c in non_pk_cols)
-                    on_conflict = f"ON CONFLICT ({conflict_cols}) DO UPDATE SET {set_clause}"
-                else:
-                    on_conflict = f"ON CONFLICT ({conflict_cols}) DO NOTHING"
 
             # Create single temp table outside loop
             temp_table = f"temp_{table}_copy"
@@ -688,13 +688,65 @@ class SQLAlchemyDatabase:
                 cursor.copy_expert(f'COPY "{temp_table}" ({non_id_col_list}) FROM STDIN WITH CSV HEADER', buf)
                 total_times["copy_data"] += time.time() - start
 
-                # 4) Time INSERT operation
+                # 4) Time INSERT/UPDATE operation
                 start = time.time()
-                insert_sql = (
-                    f'INSERT INTO "{table}" ({non_id_col_list}) SELECT {non_id_col_list} FROM "{temp_table}" '
-                    + on_conflict
-                )
-                cursor.execute(insert_sql)
+                
+                if use_on_conflict:
+                    # Traditional approach for RAISE and IGNORE modes
+                    insert_sql = (
+                        f'INSERT INTO "{table}" ({non_id_col_list}) SELECT {non_id_col_list} FROM "{temp_table}" '
+                        + on_conflict
+                    )
+                    cursor.execute(insert_sql)
+                else:
+                    # DELETE + INSERT optimization for UPDATE mode
+                    pk_col_list = ", ".join(f'"{c}"' for c in pks)
+                    
+                    if non_pk_cols:
+                        # 1) Create temp table with rows that need updating
+                        rows_to_update_table = f"temp_rows_to_update_{batch_count}"
+                        # Create properly qualified column list for SELECT
+                        qualified_non_id_col_list = ", ".join(f't."{c}"' for c in non_id_cols)
+                        cursor.execute(
+                            f'CREATE TEMP TABLE "{rows_to_update_table}" AS '  # noqa: S608
+                            f'SELECT {qualified_non_id_col_list} FROM "{temp_table}" t '
+                            f'INNER JOIN "{table}" r ON {" AND ".join(f"t.\"{c}\" = r.\"{c}\"" for c in pks)}'
+                        )
+                        
+                        # 2) Delete existing rows that will be replaced
+                        cursor.execute(
+                            f'DELETE FROM "{table}" '  # noqa: S608
+                            f'WHERE ({pk_col_list}) IN ('
+                            f'SELECT {pk_col_list} FROM "{rows_to_update_table}"'
+                            f')'
+                        )
+                        
+                        # 3) Insert new rows (those not in existing table)
+                        cursor.execute(
+                            f'INSERT INTO "{table}" ({non_id_col_list}) '  # noqa: S608
+                            f'SELECT {qualified_non_id_col_list} FROM "{temp_table}" t '
+                            f'LEFT JOIN "{rows_to_update_table}" u ON {" AND ".join(f"t.\"{c}\" = u.\"{c}\"" for c in pks)} '
+                            f'WHERE u."{pks[0]}" IS NULL'
+                        )
+                        
+                        # 4) Insert updated rows
+                        cursor.execute(
+                            f'INSERT INTO "{table}" ({non_id_col_list}) '  # noqa: S608
+                            f'SELECT {non_id_col_list} FROM "{rows_to_update_table}"'
+                        )
+                        
+                        # 5) Clean up update temp table
+                        cursor.execute(f'DROP TABLE IF EXISTS "{rows_to_update_table}"')
+                    else:
+                        # If no non-pk columns, just insert new records (ignore existing)
+                        qualified_non_id_col_list = ", ".join(f't."{c}"' for c in non_id_cols)
+                        cursor.execute(
+                            f'INSERT INTO "{table}" ({non_id_col_list}) '  # noqa: S608
+                            f'SELECT {qualified_non_id_col_list} FROM "{temp_table}" t '
+                            f'LEFT JOIN "{table}" r ON {" AND ".join(f"t.\"{c}\" = r.\"{c}\"" for c in pks)} '
+                            f'WHERE r."{pks[0]}" IS NULL'
+                        )
+                
                 total_times["insert"] += time.time() - start
 
                 # 5) Time ID fetching
@@ -997,8 +1049,8 @@ def _copy_worker_static(
             )
             cursor.execute(insert_sql)
 
-        else:  # UPDATE mode
-            # First insert new records
+        else:  # UPDATE mode - Use DELETE + INSERT optimization
+            # First insert new records (same as before)
             insert_sql = (
                 f'INSERT INTO "{table}" ({non_id_col_list}) '  # noqa: S608
                 f'SELECT {non_id_col_list} FROM "{temp_table}" AS t '
@@ -1006,17 +1058,35 @@ def _copy_worker_static(
             )
             cursor.execute(insert_sql)
 
-            # Then update existing records (only if there are non-pk columns to update)
+            # For existing records: DELETE then INSERT (faster than UPDATE)
             non_pk_cols = [c for c in cols if c not in pks]
             if non_pk_cols:
-                set_clause = ", ".join(f'"{c}" = t."{c}"' for c in non_pk_cols)
-                update_sql = (
-                    f'UPDATE "{table}" AS r SET {set_clause} '  # noqa: S608
-                    f'FROM "{temp_table}" AS t '
-                    f"WHERE {' AND '.join(f'r."{c}" = t."{c}"' for c in pks)} "
-                    f'AND EXISTS (SELECT 1 FROM "{existing_keys_table}" AS e WHERE {" AND ".join(f't."{c}" = e."{c}"' for c in pks)})'
+                # 1) Create temp table with rows that need updating (exist in both tables)
+                rows_to_update_table = f"temp_rows_to_update_worker_{offset}"
+                # Create properly qualified column list for SELECT
+                qualified_non_id_col_list = ", ".join(f't."{c}"' for c in non_id_cols)
+                cursor.execute(
+                    f'CREATE TEMP TABLE "{rows_to_update_table}" AS '  # noqa: S608
+                    f'SELECT {qualified_non_id_col_list} FROM "{temp_table}" t '
+                    f'INNER JOIN "{table}" r ON {" AND ".join(f"t.\"{c}\" = r.\"{c}\"" for c in pks)}'
                 )
-                cursor.execute(update_sql)
+                
+                # 2) Delete existing rows that will be replaced
+                cursor.execute(
+                    f'DELETE FROM "{table}" '  # noqa: S608
+                    f'WHERE ({pk_col_list}) IN ('
+                    f'SELECT {pk_col_list} FROM "{rows_to_update_table}"'
+                    f')'
+                )
+                
+                # 3) Insert updated rows from temp table
+                cursor.execute(
+                    f'INSERT INTO "{table}" ({non_id_col_list}) '  # noqa: S608
+                    f'SELECT {non_id_col_list} FROM "{rows_to_update_table}"'
+                )
+                
+                # 4) Clean up update temp table
+                cursor.execute(f'DROP TABLE IF EXISTS "{rows_to_update_table}"')
 
         # 4) Optionally retrieve IDs
         id_df = None
