@@ -791,7 +791,7 @@ class Predictions(Base):
         "datapointID",
         BigInteger,
         ForeignKey(
-            'datapoints."ID"',
+            "datapoints.ID",
             name="link_to_datapoints",
             onupdate="RESTRICT",
             ondelete="CASCADE",
