@@ -315,6 +315,7 @@ class DataLoader:
             )
             .drop("category")
             .rename({"product": "name"})
+            .unique()
         )
 
         # Use bulk insertion method
@@ -391,7 +392,6 @@ class DataLoader:
                 product_ids.drop("companyID").rename({"name": "product", "ID": "productID"}), on="product", how="left"
             )
         )
-
         time_sku_combinations = time_sku_data.select(["storeID", "productID"])
 
         # Write to SKU table
@@ -432,13 +432,21 @@ class DataLoader:
             .select(["skuID", "dateID"])
         )
 
+        if len(time_sku_data) <= 1_000_000:
+            num_workers = 1
+        elif len(time_sku_data) < 4_000_000:
+            num_workers = 2
+        elif len(time_sku_data) >= 4_000_000:
+            num_workers = 4
+
+
         # Write to Datapoints table
         result = db.handle_insertion_multi_line(
             model_class=Datapoints,
             data=time_sku_data,
             mode=self.insertion_mode,
             returning_id=True,
-            parallel_workers=4,
+            parallel_workers=num_workers,
         )
 
         logger.info("Writing datapoints to db complete, got %d datapoint IDs back", len(result))
@@ -492,12 +500,19 @@ class DataLoader:
             .select(["datapointID", "featureID", "value"])
         )
 
+        if len(time_sku_data) <= 1_000_000:
+            num_workers = 1
+        elif len(time_sku_data) < 4_000_000:
+            num_workers = 2
+        elif len(time_sku_data) >= 4_000_000:
+            num_workers = 4
+
         db.handle_insertion_multi_line(
             model_class=TimeSkuFeatures,
             data=time_sku_data,
             mode=self.insertion_mode,
             returning_id=False,
-            parallel_workers=4,
+            parallel_workers=num_workers,
         )
 
         logger.info("Writing time-sku features to the database complete, wrote %d rows", len(time_sku_data))
@@ -533,12 +548,20 @@ class DataLoader:
             }
         )
 
+
+        if len(flag_description_map) <= 1_000_000:
+            num_workers = 1
+        elif len(flag_description_map) < 4_000_000:
+            num_workers = 2
+        elif len(flag_description_map) >= 4_000_000:
+            num_workers = 4
+
         feature_description_result = db.handle_insertion_multi_line(
             model_class=FeatureDescriptions,
             data=flag_description_map,
             mode=self.insertion_mode,
             returning_id=True,
-            parallel_workers=4,
+            parallel_workers=num_workers,
         )
 
         # Prepare flags
@@ -560,13 +583,20 @@ class DataLoader:
             .select(["datapointID", "featureID", "value"])
         )
 
+        if len(flags) <= 1_000_000:
+            num_workers = 1
+        elif len(flags) < 4_000_000:
+            num_workers = 2
+        elif len(flags) >= 4_000_000:
+            num_workers = 4
+
         # Write flags to the database
         db.handle_insertion_multi_line(
             model_class=TimeSkuFeatures,  # Assuming TimeSkuFeatures can also handle flags
             data=flags,
             mode=self.insertion_mode,
             returning_id=False,
-            parallel_workers=4,
+            parallel_workers=num_workers,
         )
 
         logger.info("Writing flags to the database complete, wrote %d rows", len(flags))
@@ -651,6 +681,13 @@ class DataLoader:
             how="left",
         )
 
+        if len(feature_map) <= 1_000_000:
+            num_workers = 1
+        elif len(feature_map) < 4_000_000:
+            num_workers = 2
+        elif len(feature_map) >= 4_000_000:
+            num_workers = 4
+
         # Optionally join with dates if provided
         if dates_df is not None:
             feature_map = feature_map.join(dates_df, on="date", how="left")
@@ -662,7 +699,7 @@ class DataLoader:
             data=feature_map,
             mode=self.insertion_mode,
             returning_id=False,
-            parallel_workers=2,  # anything not time-sku should use 2 worker
+            parallel_workers=num_workers, 
         )
 
         logger.info("Writing %s features to the database complete, wrote %d rows", feature_type, len(feature_map))
