@@ -795,9 +795,12 @@ class DataValidator:
             msg = f"Categorical features must have non-null levels: {invalid_features}"
             raise ValueError(msg)
 
+        text_features = description.filter(pl.col("var_type") == VariableTypes.TEXT.value)
+
         # Determine required value column type based on categorical features
         has_categorical = not categorical_features.is_empty()
-        expected_value_type = pl.Utf8 if has_categorical else pl.Float64
+        has_text = not text_features.is_empty()
+        expected_value_type = pl.Utf8 if has_categorical or has_text else pl.Float64
 
         # Validate value column type
         actual_value_type = data["value"].dtype
@@ -849,21 +852,17 @@ class DataValidator:
 
         if categorical_features.is_empty():
             return  # No categorical features to validate
-        
+
         # Create allowed values DataFrame
         levels_df = (
-            categorical_features
-            .select("name", "levels")
+            categorical_features.select("name", "levels")
             .explode("levels")
             .rename({"name": "feature", "levels": "allowed_value"})
         )
 
         # Get only categorical feature data by joining with categorical_features
         categorical_data = data.join(
-            categorical_features.select("name"), 
-            left_on="feature", 
-            right_on="name", 
-            how="inner"
+            categorical_features.select("name"), left_on="feature", right_on="name", how="inner"
         ).filter(pl.col("value").is_not_null())
 
         if categorical_data.is_empty():
@@ -871,16 +870,15 @@ class DataValidator:
 
         # Find invalid values using left join and check for nulls
         invalid_data = (
-            categorical_data
-            .join(levels_df, left_on=["feature", "value"], right_on=["feature", "allowed_value"], how="left")
-            .filter(pl.col("feature").is_null())  # Invalid values will have null allowed_value
+            categorical_data.join(
+                levels_df, left_on=["feature", "value"], right_on=["feature", "allowed_value"], how="left"
+            ).filter(pl.col("feature").is_null())  # Invalid values will have null allowed_value
         )
 
         if not invalid_data.is_empty():
             # Join back with categorical_features to get the levels for error message
             invalid_summary = (
-                invalid_data
-                .join(categorical_features.select(["name", "levels"]), left_on="feature", right_on="name")
+                invalid_data.join(categorical_features.select(["name", "levels"]), left_on="feature", right_on="name")
                 .group_by(["feature", "levels"])
                 .agg(pl.col("value").unique().head(10).alias("invalid_values"))
                 .sort("feature")
@@ -900,18 +898,15 @@ class DataValidator:
 
             msg = "Categorical features have invalid values:\n" + "\n".join(error_details)
             raise ValueError(msg)
-    
+
     def _validate_text_feature_values(self, description: pl.DataFrame, data: pl.DataFrame) -> None:
         """Validate that text feature values are strings or null."""
-
         if description.filter(pl.col("var_type") == VariableTypes.TEXT.value).is_empty():
             return  # No text features to validate
 
         # If text features present, the entire value columns must be String
         if data.select(pl.col("value")).dtypes[0] != pl.Utf8:
             raise TypeError("Text feature values must be of type Utf8")
-            
-
 
     def _validate_binary_feature_values(self, description: pl.DataFrame, data: pl.DataFrame) -> None:
         """Validate that binary feature values are only 0 or 1."""
